package edu.travel.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;


/**
 * JwtUtil 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/3/14
 */

public class JwtUtil {
    private static final String SECRET_KEY = "Xk7f38@LsP9zU!d#2b9j6vFqY5Rm8zE9Fj1oH@7tR"; // 32+字符
    private static final long EXPIRATION_TIME = 1000 * 60 * 60 * 24; // 24 小时

    /**
     * 生成 JWT Token
     *
     * @param user 用户对象
     * @return 生成的 Token
     */
    public static <T> String generateJwtToken(T user) {
        return JWT.create()
                .withSubject(user.getClass().getSimpleName())  // 主题：类名
                .withClaim("userInfo", JSON.toJSONString(user)) // 存储用户数据
                .withIssuedAt(new Date())  // 签发时间
                .withExpiresAt(new Date(System.currentTimeMillis() + EXPIRATION_TIME)) // 过期时间
                .sign(Algorithm.HMAC256(SECRET_KEY)); // HMAC SHA256 签名
    }

    /**
     * 解析 JWT Token 并获取用户对象
     *
     * @param token JWT Token
     * @param clazz 目标对象的 Class
     * @return 解析出的用户对象
     */
    public static <T> T parseJwtToken(String token, Class<T> clazz) {
        DecodedJWT decodedJWT = JWT.require(Algorithm.HMAC256(SECRET_KEY))
                .build()
                .verify(token);

        String userJson = decodedJWT.getClaim("userInfo").asString();
        return JSON.parseObject(userJson, clazz);
    }

    /**
     * 从请求头获取 Token 并解析成用户对象
     *
     * @param request HTTP 请求
     * @param clazz   目标对象的 Class
     * @return 解析出的用户对象
     */
    public static <T> T getUserFromRequest(HttpServletRequest request, Class<T> clazz) {
        String token = request.getHeader("Authorization");
        if (StringUtils.isBlank(token) || token.equals("null")) {
            throw new RuntimeException("Token 不能为空");
        }

        token = token.replace("Bearer ", ""); // 去掉 "Bearer " 前缀
        return parseJwtToken(token, clazz);
    }
}
