package edu.travel.util;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

/**
 * UserContext 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/3/14
 */

@Component
public class UserContext<T> {

    private static RedisTemplate<String, String> redisTemplate;

    @Autowired
    public void setRedisTemplate(RedisTemplate<String, String> redisTemplate) {
        UserContext.redisTemplate = redisTemplate;
    }

    /**
     * 获取当前 HTTP 请求
     */
    private static HttpServletRequest getCurrentRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return attributes != null ? attributes.getRequest() : null;
    }

    /**
     * 获取当前用户（泛型）
     */
    public static <T> T getCurrentUser(Class<T> clazz) {
        HttpServletRequest request = getCurrentRequest();
        if (request == null) {
            return null;
        }

        String token = request.getHeader("Authorization");
        if (StringUtils.isBlank(token) || "null".equals(token)) {
            return null;
        }

        try {
            // 解析 JWT
            T user = JwtUtil.parseJwtToken(token.replace("Bearer ", ""), clazz);
            if (user == null) {
                return null;
            }

            // 获取用户的唯一标识（假设所有用户类型都有 getPhone 方法）
            String phone = getPhoneFromUser(user);
            if (StringUtils.isBlank(phone)) {
                return null;
            }

            // 从 Redis 获取用户信息
            String redisUserJson = redisTemplate.opsForValue().get(phone + "_user_info");
            if (StringUtils.isNotBlank(redisUserJson)) {
                return JSON.parseObject(redisUserJson, clazz); // 返回用户信息
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * 通过反射从用户对象获取唯一标识（

     */
    private static <T> String getPhoneFromUser(T user) {
        try {
            Method getPhoneMethod = user.getClass().getMethod("getPhone");
            return (String) getPhoneMethod.invoke(user);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
