package edu.travel.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import edu.travel.dto.FeedRecordCreateDTO;
import edu.travel.exception.BaseException;
import edu.travel.service.BtFeedRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import edu.travel.entity.BtFeedRecord;
import edu.travel.mapper.BtFeedRecordMapper;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

@Service
public class BtFeedRecordServiceImpl extends ServiceImpl<BtFeedRecordMapper, BtFeedRecord> implements BtFeedRecordService {
    @Autowired
    private BtFeedRecordMapper btFeedRecordMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createRecord(FeedRecordCreateDTO dto, String project) {
        // 校验时间顺序
        /*if (dto.getEndTime().before(dto.getStartTime())) {
            throw new BaseException("结束时间不能早于开始时间");
        }*/

        // 类型相关校验
        switch (dto.getType()) {
            case 1: // 喂奶
                validateNursing(dto);
                break;
            case 2: // 辅食
                validateComplementary(dto);
                break;
            case 3: // 尿布
                validateDiaper(dto);
                break;
            case 5: // 吸奶
                validateMilkExpression(dto);
                break;
            case 6://洗澡
                validateXizao(dto);
                break;
            case 7://游泳
                validateSwimming(dto);
                 break;
        }
        // DTO转Entity
        BtFeedRecord record = new BtFeedRecord();
                record.setType(dto.getType());
                record .setNurseMethod(dto.getNurseMethod());
                record.setNurseAmount(dto.getNurseAmount());
                record.setComplementaryName(dto.getComplementaryName());
                record.setHasStool(dto.getHasStool());
                record.setMilkOutput(dto.getMilkOutput());
                record.setStartTime(dto.getStartTime());
        if (dto.getEndTime() != null) {
            record.setEndTime(dto.getEndTime());
        }else {record.setEndTime(new Date());}
                record.setRemark(dto.getRemark());
                record.setImages(dto.getImages());
                record.setProject(project);
                record.setRemark(dto.getRemark());
                record.setCreateTime(new Date());
                record.setCreateUserId(String.valueOf(1));
                record.setUpdateUserId(String.valueOf(1));
                record.setDeleteFlag(0);

        int rows = btFeedRecordMapper.insert(record);
        if (rows != 1) {
            throw new BaseException("记录保存失败");
        }
    }

    private void validateXizao(FeedRecordCreateDTO dto) {
        if (dto.getStartTime() == null) {
            throw new BaseException("开始时间不能为空");
        }
    }

    private void validateSwimming(FeedRecordCreateDTO dto) {
        if (dto.getStartTime() == null) {
            throw new BaseException("开始时间不能为空");
        }
    }

    private void validateNursing(FeedRecordCreateDTO dto) {
        if (dto.getNurseMethod() == null) {
            throw new BaseException("喂奶方式不能为空");
        }
     /*   if (dto.getNurseMethod() == 1 && (dto.getStartTime() == null||dto.getEndTime() == null )) {
            throw new BaseException("喂奶方式为纯喂奶时开始时间或结束时间不能为空");
        }*/
        if ((dto.getNurseMethod() == 2 || dto.getNurseMethod() == 3)
                && (dto.getNurseAmount() == null || dto.getNurseAmount() <= 0)) {
            throw new BaseException("喂奶量必须大于0");
        }
    }

    private void validateComplementary(FeedRecordCreateDTO dto) {
        if (StringUtils.isBlank(dto.getComplementaryName())) {
            throw new BaseException("辅食名称不能为空");
        }
    }
    private void validateDiaper(FeedRecordCreateDTO dto) {
        if (dto.getHasStool() == null) {
            throw new BaseException("必须指定是否有粪便");
        }
    }
    private void validateMilkExpression(FeedRecordCreateDTO dto) {
        if (dto.getMilkOutput() == null || dto.getMilkOutput() < 0) {
            throw new BaseException("吸奶量必须大于等于0");
        }
    }
}
