package edu.travel.filter;

import com.alibaba.fastjson.JSON;
import edu.travel.util.JwtUtil;
import edu.travel.vo.BtAccountInfoVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * LoginFilter 类。
 * <p>
 * 描述：拦截请求，解析 JWT 并从 Redis 获取用户信息
 *
 * @author huangwenwen
 * @date 2025/3/11
 */

@Component
public class LoginFilter implements Filter {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {

        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;

        String requestURI = request.getRequestURI();

        if (requestURI.startsWith("/baby/login")) {
            filterChain.doFilter(request, response);
            return;
        }

        String token = request.getHeader("Authorization");

        if (StringUtils.isBlank(token) || "null".equals(token)) {
            filterChain.doFilter(request, response);
            return;
        }

        try {
            BtAccountInfoVo user = JwtUtil.parseJwtToken(token.replace("Bearer ", ""), BtAccountInfoVo.class);
            if (user == null) {
                filterChain.doFilter(request, response);
                return;
            }

            String redisUserJson = redisTemplate.opsForValue().get(user.getPhone() + "_user_info");
            if (StringUtils.isNotBlank(redisUserJson)) {
                BtAccountInfoVo redisUser = JSON.parseObject(redisUserJson, BtAccountInfoVo.class);
                if (redisUser != null) {
                    request.setAttribute("currentUser", redisUser);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        filterChain.doFilter(request, response);
    }
}
