package edu.travel.web;

import edu.travel.dto.BabyInfoDTO;
import edu.travel.dto.BabyQueryDTO;
import edu.travel.resp.BaseResponse;
import edu.travel.service.BtBabyInfoService;
import edu.travel.vo.BabyDetailVO;
import edu.travel.vo.BabyInfoVO;
import edu.travel.vo.BabyListItemVO;
import edu.travel.vo.PageResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

/**
 * @author Survive
 * @date 2025/3/13
 * @description TODO 宝宝信息接口
 */
@RestController
@RequestMapping("/baby/info")
public class BabyInfoController {

    @Autowired
    private BtBabyInfoService babyService;
    
    @GetMapping("/list")
    public BaseResponse<PageResult<BabyListItemVO>> listBabies(
            @Valid BabyQueryDTO queryDTO,
            @RequestHeader("Authorization") String token
    ) {
//        Long currentUserId = SecurityUtils.getCurrentUserId();
        PageResult<BabyListItemVO> babyListItemVOPageResult = babyService.listBabiesByPage(queryDTO, 1L);
        return new BaseResponse<>(200, "success", babyListItemVOPageResult);
    }

    /**
     * 获取宝宝信息
     * @param babyId
     * @return
     */
    @GetMapping("/getDetail/{babyId}")
    public BaseResponse<BabyDetailVO> getBabyDetail(@PathVariable Long babyId
//                                                    @RequestHeader("Authorization") String token
    ) {
        BabyDetailVO babyDetailVO = babyService.getBabyInfoDetail(babyId);
        return new BaseResponse<>(200, "success", babyDetailVO);
    }

    /**
     * 创建宝宝信息
     * @param dto
     * @return
     */
    @PostMapping("/add")
    public BaseResponse<BabyInfoVO> createBaby(HttpServletRequest request, @RequestBody @Valid BabyInfoDTO dto) {
//        Long currentUserId = SecurityUtils.getCurrentUserId();
//        babyService.createBaby(dto, currentUserId);
        return new BaseResponse<>(200, "success", babyService.addBaby(dto, 1L));
    }


    /**
     * 更新宝宝信息
     * @param dto
     * @return
     */
    @PutMapping("/update")
    public BaseResponse<BabyInfoVO> updateBabyInfo(@RequestBody @Valid BabyInfoDTO dto) {
//        Long currentUserId = SecurityUtils.getCurrentUserId();
        return new BaseResponse<>(200, "success", babyService.updateBabyInfo(dto, 1L));
    }

    /**
     * 删除宝宝信息
     * @param babyId
     * @return
     */
    @DeleteMapping("/delete/{babyId}")
    public BaseResponse deleteBabyInfo(@PathVariable Long babyId) {
//        Long currentUserId = SecurityUtils.getCurrentUserId();
        String s = babyService.deleteBabyWithValidation(babyId, 1L);
        return new BaseResponse<>(200, "success", s);
    }


}